/**
********************************************************************************
* @file       M6678E_I2C.c
* @brief      ʵM6678E I2Cõĸֽӿں
* @version    V1.0
* @date       2024.1.9
* @attention
* \par
* ==============================================================================
* @n   (C) DSP֧Ŷ.
********************************************************************************
*/
#include "M6678E_I2C.h"
#include "M6678E_PSC.h"

/** @addtogroup I2C_FUNCTION
 @{ */
extern int I2CMDR_Switch;
/**
********************************************************************************
 *   @n@b void I2C_CLK_Init(I2C_TypeDef *I2C_Dev, uint16_t code_frq, uint16_t i2c_out_frq)
 *
 *   @b ܣ
 *   @n  I2Cʱ
 *
 *   @b βΣ
 *   @verbatim
 *		I2C_Dev  		I2C豸	DEV_I2C0DEV_I2C1
 * 		code_frq 		ϵͳƵʱMHz
 * 		i2c_out_frq 	I2CʱkHz
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n ֻʱӣûģʽ
 *******************************************************************************
 */
void I2C_CLK_Init(I2C_TypeDef *I2C_Dev, uint16_t code_frq, uint16_t i2c_out_frq) 
{
	I2C_Dev->I2CMDR = 0x10;	// ʼI2CͷI2C
	uint32_t div_l,div_h,div_num,i2c_clk,div_psc;
	double i2c_clk_actual;
	uint8_t d1, d2;

//	div_psc = (code_frq/4)/12;      // С13.3M
	div_psc = (code_frq/2)/12;
	
	if(div_psc == 0){
		d1 = 7;
		d2 = 9;
	}else if(div_psc == 1){
		d1 = 6;
		d2 = 7;
	}else{
		d1 = 5;
		d2 = 5;
	}

	//  i2c_clk = (code_frq/4)/((div_psc+1)*2)*1000;  
    	i2c_clk = (code_frq*1000)/((div_psc+1)*4);

//  div_num = (i2c_clk/i2c_out_frq)*3/4;
    	div_num = i2c_clk/i2c_out_frq;

	if(div_num > (d1+d2))
	{
		div_num -= (d1+d2);
		div_l = div_num/2;
		div_h = div_num/2;
		if((div_num%2) != 0) div_h += 1;
		if(div_l == 0) div_l = 1;
		if(div_h == 0) div_h = 1;
	}
	else
	{
		div_l = 1;
		div_h = 1;
	}
    	i2c_clk_actual=(float)i2c_clk/(div_l+d1+div_h+d2);//XT4000100 khz  ʵʾ102khz


	printf("main_clk: %dMHz, clk_set:%dkHz, i2c_clk_actual:%0.2f kHz.\n",
			//	code_frq, i2c_out_frq,(float)i2c_clk/(div_l+d1+div_h+d2));
				code_frq, i2c_out_frq,(double)i2c_clk/(div_l+d1+div_h+d2));
 	printf("i2c_clk_actual:%0.2f /%d kHz.\n",(float)i2c_clk,(div_l+d1+div_h+d2));//136khz

	I2C_Dev->I2CPSC = div_psc;	// ԤƵ
	I2C_Dev->I2CCLKL = div_l;
	I2C_Dev->I2CCLKH = div_h;
}

/**
********************************************************************************
 *   @n@b void I2C_Write(I2C_TypeDef *I2C_Dev, uint8_t I2C_address, uint32_t ByteCount, uint8_t *pBuffer)
 *
 *   @b ܣ
 *   @n  дָ
 *
 *   @b βΣ
 *   @verbatim
 *		I2C_Dev 		I2C豸	DEV_I2C0DEV_I2C1DEV_I2C2
 * 		I2C_address 	豸ĵַ
 * 		pBuffer			͵
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */

void I2C_Write(I2C_TypeDef *I2C_Dev, uint8_t I2C_address, uint32_t ByteCount, uint8_t *pBuffer)
{
	uint32_t i = 0;
	/****************Ϊģʽ****************/
	I2C_Dev->I2CSTR |= I2C_Dev->I2CSTR;				// ״̬Ĵ
	I2C_Dev->I2CMDR = 0;
	I2C_Dev->I2CMDR |= (1<<14)|(1<<13)|(1<<11)|(1<<10)|(1<<9);	// FREE MST TRX
	I2C_Dev->I2CSAR = I2C_address;					// 7λӻַ
	I2C_Dev->I2COAR = 0x60;                     			// ַ
	I2C_Dev->I2CCNT = ByteCount;					// ͵
	I2C_Dev->I2CMDR |= (1<<5);					// ʹI2C
//	while((I2C_Dev->I2CSTR & (1<<12)));				// ȴBB
	
	// 
	for(i=0; i<ByteCount; i++)
	{
 		I2C_Dev->I2CDXR = pBuffer[i];
		while(!(I2C_Dev->I2CSTR & 0x10));			// ȴ
		I2C_Dev->I2CSTR &= 0x10;				// ñ־λ I2C_DXRдʱԶ
	}

}

/**
********************************************************************************
 *   @n@b void I2C_Read(I2C_TypeDef *I2C_Dev, uint8_t I2C_address, uint32_t ByteCount, uint8_t *pBuffer)
 *
 *   @b ܣ
 *   @n  ȡָ
 *
 *   @b βΣ
 *   @verbatim
 *		I2C_Dev  		I2C豸	DEV_I2C0DEV_I2C1
 * 		I2C_address 		豸ĵַ
 * 		ByteCount 		ȡֽ
 * 		pBuffer 		ȡ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void I2C_Read(I2C_TypeDef *I2C_Dev, uint8_t I2C_address, uint32_t ByteCount, uint8_t *pBuffer)
{
	uint32_t i = 0;
	/****************Ϊȡģʽ****************/
	I2C_Dev->I2CSTR |= I2C_Dev->I2CSTR;				// ״̬Ĵ
	I2C_Dev->I2CMDR = 0;
	I2C_Dev->I2CMDR |= (1<<14)|(1<<13)|(1<<11)|(1<<10);		// FREE MST TRX=0
	I2C_Dev->I2CSAR = I2C_address;					// 7λӻַ
//	I2C_Dev->I2COAR = 0x60;
	I2C_Dev->I2CCNT = ByteCount;					// յ
	I2C_Dev->I2CMDR |= (1<<5);					// ʹI2C

	/**************** ʼ ****************/
	for(i=0; i<ByteCount; i++)
	{
		while(!(I2C_Dev->I2CSTR & 0x08));
		pBuffer[i] = I2C_Dev->I2CDRR;
	}
}

/**
********************************************************************************
 *   @n@b int I2C_Write_Read_Byte(I2C_TypeDef *I2C_Dev, unsigned char w_dat)
 *
 *   @b ܣ
 *   @n  BLDõĶд1ֽڲ֤
 *
 *   @b βΣ
 *   @verbatim
 *		I2C_Dev 		I2C豸	DEV_I2C0DEV_I2C1
 *		w_dat 	 		дֵ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n  -1 ʧ , 0 ɹ
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
int I2C_Write_Read_Byte(I2C_TypeDef *I2C_Dev, unsigned char w_dat)
{
	unsigned char temp = 0;

	I2C_Dev->I2CDXR = w_dat;
	while(!(I2C_Dev->I2CSTR & 0x10));	//  ȴ
	I2C_Dev->I2CSTR &= 0x10;		// ñ־λ
	while(!(I2C_Dev->I2CSTR & 0x08));	// ȴȡ
	temp = I2C_Dev->I2CDRR;
	I2C_Dev->I2CSTR &= 0x08;		// ñ־λ д1

	if(temp != w_dat)
		return -1;			// ʧ
	return 0;				// Գɹ
}

/**
********************************************************************************
 *   @n@b void loop_test(I2C_TypeDef *I2C_Dev)
 *
 *   @b ܣ
 *   @n  ΪػģʽӡԽ
 *
 *   @b βΣ
 *   @verbatim
 *		I2C_Dev   I2C豸	DEV_I2C0DEV_I2C1
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void loop_test(I2C_TypeDef *I2C_Dev)
{
	unsigned int i = 0;

	I2C_Dev->I2CSTR |= I2C_Dev->I2CSTR;		// ״̬Ĵ
	I2C_Dev->I2CMDR = 0;
	// STT STP FREE MST TRX DLB
	I2C_Dev->I2CMDR = (1<<14)|(1<<13)|(1<<11)|(1<<10)|(1<<9)|(1<<6);

	I2C_Dev->I2CCNT = 1024;	// дֽ
	I2C_Dev->I2CSAR = 0x60;	// дַ
	I2C_Dev->I2COAR = 0x60;

	I2C_Dev->I2CMDR |= (1<<5);			// ʹI2C
//	while((I2C_Dev->I2CSTR & (1<<12)));		// ȴBB
	for(i=0; i<1024; i++)
	{
		if(I2C_Write_Read_Byte(I2C_Dev, i%256) != 0)
        {
		    printf("loop_test fail! i:%d\n",i);
		    break;
        }
	}
	if(i == 1024)
		printf("loop_test successful!\n");
	else
		printf("loop_test fail!\n");
}


void Data_cmpare(unsigned char *Srcdata_addr,  unsigned char *Dstdata_addr, unsigned int size)
{
	volatile unsigned int i=0, flag=0;
	volatile unsigned int src_data=0,dst_data=0;
	volatile unsigned char *src_addr, *dst_addr;
	volatile unsigned int  temp_size=0;
	src_addr = Srcdata_addr;
	dst_addr = Dstdata_addr;
	temp_size = size;
	for(i=0;i<temp_size;i++)				//ѭȽϽ
	{
		src_data = *src_addr++;
		dst_data = *dst_addr++;
		if(src_data != dst_data)
		{
			flag = 1;
			break;
		}
	}

	if(flag==1)
	{
		printf("дݱȽϽ!!!\n");
	}
	else
	{
		printf("дݱȽϽȷ!!!\n");
	}
}

/**
@}
*/
