/**
********************************************************************************
* @file       M6678E_MEMPROT.c
* @brief      MEMPROTӿں
* @version    V1.0
* @date       2022.07.26
* @attention
* \par
* ==============================================================================
* @n   (C) DSP֧Ŷ.
********************************************************************************
*/
#include <stdio.h>
#include <string.h>
#include "M6678E_MEMPROT.h"

/** @addtogroup MEMPROT_FUNCTION
 @{ */

/*======================memory protection==============================*/
/*code and data related to CorePac MPAX configuration should be allocated in LL2*/
#pragma DATA_SECTION(mem_prot_key,".far:Core_MPAX")
unsigned int mem_prot_key[4] = {0xBFBFBFBF, 0xFE29, 3, 4};

/**
********************************************************************************
 *   @n@b unsigned int MEMPROT_getL1PFaultAddress()
 *
 *   @b ܣ
 *   @n ȡL1P洢ַ
 *
 *   @b βΣ
 *   @verbatim
 *      
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n  洢ַ
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
unsigned int MEMPROT_getL1PFaultAddress()
{
	return hL1pMemprotRegs->L1PMPFAR;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_clearL1PFault()
 *
 *   @b ܣ
 *   @n L1P洢Ĵ
 *
 *   @b βΣ
 *   @verbatim
 *      mpfsr 洢üĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_clearL1PFault()
{
	hL1pMemprotRegs->L1PMPFCR = 1;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_getL1PFaultStatus(MEMPROT_MPFSR * mpfsr)
 *
 *   @b ܣ
 *   @n ȡL1P洢״̬
 *
 *   @b βΣ
 *   @verbatim
 *      mpfsr 洢üĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_getL1PFaultStatus(MEMPROT_MPFSR * mpfsr)
{
    unsigned int tmp = 0;
    tmp = hL1pMemprotRegs->L1PMPFSR;
	mpfsr->fid = (tmp & 0x0000FE00) >> 9;
	mpfsr->local = (tmp & 0x00000100) >> 8;
	mpfsr->sr = (tmp & 0x00000020) >> 5;
	mpfsr->sw = (tmp & 0x00000010) >> 4;
	mpfsr->ur = (tmp & 0x00000004) >> 2;
	mpfsr->uw = (tmp & 0x00000002) >> 1;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_setL1PAttributes(unsigned int index,MEMPROT_MPPA * mppa)
 *
 *   @b ܣ
 *   @n L1P洢ҳԼĴ
 *
 *   @b βΣ
 *   @verbatim
 *      index  洢ҳ0~15Ӧҳ16~31
 *      mppa   洢ҳԼĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_setL1PAttributes(unsigned int index,MEMPROT_MPPA * mppa)
{
	hL1pMemprotRegs->L1PMPPA[index] = (mppa->aid5 << 15) |
									  (mppa->aid4 << 14) |
									  (mppa->aid3 << 13) |
									  (mppa->aid2 << 12) |
									  (mppa->aid1 << 11) |
									  (mppa->aid0 << 10) |
									  (mppa->aidx << 9 ) |
									  (mppa->local << 8) |
									  (mppa->sr << 5) |
									  (mppa->sw << 4) |
									  (mppa->sx << 3) |
									  (mppa->ur << 2) |
									  (mppa->uw << 1) |
									  (mppa->ux );
}

/**
********************************************************************************
 *   @n@b void MEMPROT_getL1PAttributes(unsigned int index,MEMPROT_MPPA * mppa)
 *
 *   @b ܣ
 *   @n ȡL1P洢ҳԼĴ
 *
 *   @b βΣ
 *   @verbatim
 *      index  洢ҳ0~15Ӧҳ16~31
 *      mppa   洢ҳԼĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_getL1PAttributes(unsigned int index,MEMPROT_MPPA * mppa)
{
	unsigned int tmp = 0;
	tmp = hL1pMemprotRegs->L1PMPPA[index];
	mppa->aid5 = ((tmp & 0x8000) >> 15);
	mppa->aid4 = ((tmp & 0x4000) >> 14);
	mppa->aid3 = ((tmp & 0x2000) >> 13);
	mppa->aid2 = ((tmp & 0x1000) >> 12);
	mppa->aid1 = ((tmp & 0x800) >> 11);
	mppa->aid0 = ((tmp & 0x400) >> 10);
	mppa->aidx = ((tmp & 0x200) >> 9 );
	mppa->local= ((tmp & 0x100) >> 8);
	mppa->sr = ((tmp & 0x20) >> 5);
	mppa->sw = ((tmp & 0x10) >> 4);
	mppa->sx = ((tmp & 0x8)  >> 3);
	mppa->ur = ((tmp & 0x4)  >> 2);
	mppa->uw = ((tmp & 0x2)  >> 1);
	mppa->ux = (tmp & 0x1);
}


/**
********************************************************************************
 *   @n@b unsigned int MEMPROT_getL1DFaultAddress()
 *
 *   @b ܣ
 *   @n ȡL1D洢ַ
 *
 *   @b βΣ
 *   @verbatim
 *      
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n  洢ַ
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
unsigned int MEMPROT_getL1DFaultAddress()
{
	return hL1dMemprotRegs->L1DMPFAR;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_clearL1DFault()
 *
 *   @b ܣ
 *   @n L1D洢Ĵ
 *
 *   @b βΣ
 *   @verbatim
 *      mpfsr 洢üĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_clearL1DFault()
{
	hL1dMemprotRegs->L1DMPFCR = 1;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_getL1DFaultStatus(MEMPROT_MPFSR * mpfsr)
 *
 *   @b ܣ
 *   @n ȡL1D洢״̬
 *
 *   @b βΣ
 *   @verbatim
 *      mpfsr 洢üĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_getL1DFaultStatus(MEMPROT_MPFSR * mpfsr)
{
    unsigned int tmp = 0;

    tmp = hL1dMemprotRegs->L1DMPFSR;
	mpfsr->fid = (tmp & 0x0000FE00)>>9;
	mpfsr->local = (tmp & 0x00000100)>>8;
	mpfsr->sr = (tmp & 0x00000020)>>5;
	mpfsr->sw = (tmp & 0x00000010)>>4;
	mpfsr->ur = (tmp & 0x00000004)>>2;
	mpfsr->uw = (tmp & 0x00000002)>>1;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_setL1DAttributes(unsigned int index,MEMPROT_MPPA * mppa)
 *
 *   @b ܣ
 *   @n L1D洢ҳԼĴ
 *
 *   @b βΣ
 *   @verbatim
 *      index  洢ҳ0~15Ӧҳ16~31
 *      mppa   洢ҳԼĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_setL1DAttributes(unsigned int index,MEMPROT_MPPA * mppa)
{
	hL1dMemprotRegs->L1DMPPA[index] = (mppa->aid5 << 15) |
									  (mppa->aid4 << 14) |
									  (mppa->aid3 << 13) |
									  (mppa->aid2 << 12) |
									  (mppa->aid1 << 11) |
									  (mppa->aid0 << 10) |
									  (mppa->aidx << 9 ) |
									  (mppa->local << 8) |
									  (mppa->sr << 5) |
									  (mppa->sw << 4) |
									  (mppa->sx << 3) |
									  (mppa->ur << 2) |
									  (mppa->uw << 1) |
									  (mppa->ux );
}

/**
********************************************************************************
 *   @n@b void MEMPROT_getL1DAttributes(unsigned int index,MEMPROT_MPPA * mppa)
 *
 *   @b ܣ
 *   @n ȡL1D洢ҳԼĴ
 *
 *   @b βΣ
 *   @verbatim
 *      index  洢ҳ0~15Ӧҳ16~31
 *      mppa   洢ҳԼĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_getL1DAttributes(unsigned int index,MEMPROT_MPPA * mppa)
{
	unsigned int tmp = 0;
	tmp = hL1dMemprotRegs->L1DMPPA[index];
	mppa->aid5 = ((tmp & 0x8000) >> 15);
	mppa->aid4 = ((tmp & 0x4000) >> 14);
	mppa->aid3 = ((tmp & 0x2000) >> 13);
	mppa->aid2 = ((tmp & 0x1000) >> 12);
	mppa->aid1 = ((tmp & 0x800) >> 11);
	mppa->aid0 = ((tmp & 0x400) >> 10);
	mppa->aidx = ((tmp & 0x200) >> 9 );
	mppa->local= ((tmp & 0x100) >> 8);
	mppa->sr = ((tmp & 0x20) >> 5);
	mppa->sw = ((tmp & 0x10) >> 4);
	mppa->sx = ((tmp & 0x8)  >> 3);
	mppa->ur = ((tmp & 0x4)  >> 2);
	mppa->uw = ((tmp & 0x2)  >> 1);
	mppa->ux = (tmp & 0x1);
}



/**
********************************************************************************
 *   @n@b unsigned int MEMPROT_getL2FaultAddress()
 *
 *   @b ܣ
 *   @n ȡL2洢ַ
 *
 *   @b βΣ
 *   @verbatim
 *      
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n  洢ַ
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
unsigned int MEMPROT_getL2FaultAddress()
{
	return hL2MemprotRegs->L2MPFAR;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_clearL2Fault()
 *
 *   @b ܣ
 *   @n L2洢Ĵ
 *
 *   @b βΣ
 *   @verbatim
 *      mpfsr 洢üĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_clearL2Fault()
{
	hL2MemprotRegs->L2MPFCR = 1;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_getL2FaultStatus(MEMPROT_MPFSR * mpfsr)
 *
 *   @b ܣ
 *   @n ȡL2洢״̬
 *
 *   @b βΣ
 *   @verbatim
 *      mpfsr 洢üĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_getL2FaultStatus(MEMPROT_MPFSR * mpfsr)
{
    unsigned int tmp = 0;
    tmp = hL2MemprotRegs->L2MPFSR;
	mpfsr->fid = (tmp & 0x0000FE00) >> 9;
	mpfsr->local = (tmp & 0x00000100) >> 8;
	mpfsr->sr = (tmp & 0x00000020) >> 5;
	mpfsr->sw = (tmp & 0x00000010) >> 4;
	mpfsr->ur = (tmp & 0x00000004) >> 2;
	mpfsr->uw = (tmp & 0x00000002) >> 1;
}

/**
********************************************************************************
 *   @n@b void MEMPROT_setL2Attributes(unsigned int index,MEMPROT_MPPA * mppa)
 *
 *   @b ܣ
 *   @n L2洢ҳԼĴ
 *
 *   @b βΣ
 *   @verbatim
 *      index  洢ҳ0~15Ӧҳ16~31
 *      mppa   洢ҳԼĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_setL2Attributes(unsigned int index,MEMPROT_MPPA * mppa)
{
	hL2MemprotRegs->L2MPPA[index] = (mppa->aid5 << 15) |
									  (mppa->aid4 << 14) |
									  (mppa->aid3 << 13) |
									  (mppa->aid2 << 12) |
									  (mppa->aid1 << 11) |
									  (mppa->aid0 << 10) |
									  (mppa->aidx << 9 ) |
									  (mppa->local << 8) |
									  (mppa->sr << 5) |
									  (mppa->sw << 4) |
									  (mppa->sx << 3) |
									  (mppa->ur << 2) |
									  (mppa->uw << 1) |
									  (mppa->ux );
}

/**
********************************************************************************
 *   @n@b void MEMPROT_getL2Attributes(unsigned int index,MEMPROT_MPPA * mppa)
 *
 *   @b ܣ
 *   @n ȡL2洢ҳԼĴ
 *
 *   @b βΣ
 *   @verbatim
 *      index  洢ҳ0~15Ӧҳ16~31
 *      mppa   洢ҳԼĴĽṹ
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void MEMPROT_getL2Attributes(unsigned int index,MEMPROT_MPPA * mppa)
{
	unsigned int tmp = 0;
	tmp = hL2MemprotRegs->L2MPPA[index];
	mppa->aid5 = ((tmp & 0x8000) >> 15);
	mppa->aid4 = ((tmp & 0x4000) >> 14);
	mppa->aid3 = ((tmp & 0x2000) >> 13);
	mppa->aid2 = ((tmp & 0x1000) >> 12);
	mppa->aid1 = ((tmp & 0x800) >> 11);
	mppa->aid0 = ((tmp & 0x400) >> 10);
	mppa->aidx = ((tmp & 0x200) >> 9 );
	mppa->local= ((tmp & 0x100) >> 8);
	mppa->sr = ((tmp & 0x20) >> 5);
	mppa->sw = ((tmp & 0x10) >> 4);
	mppa->sx = ((tmp & 0x8)  >> 3);
	mppa->ur = ((tmp & 0x4)  >> 2);
	mppa->uw = ((tmp & 0x2)  >> 1);
	mppa->ux = (tmp & 0x1);
}

/**
********************************************************************************
 *   @n@b void lock_mem_prot_regs()
 *
 *   @b ܣ
 *   @n Դ洢Ĵ
 *
 *   @b βΣ
 *   @verbatim
 *      
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void lock_mem_prot_regs()
{
	unsigned int i = 0;

	/*1. Write a 1 to the KEYR field of the MPLKCMD register. This resets some internal
	status for the MPLK0 through MPLK3 registers.*/
	hMplkRegs->MPLKCMD = 0x4;

	/*2. Write the key to MPLK0 through MPLK3. All four registers must be written
	exactly once. They may be written in any order.*/
	for(i=0; i<4; i++)
	{
		hMplkRegs->MPLK[i] = mem_prot_key[i];
	}

	/*3. Write a 1 to the LOCK field of the MPLKCMD register. This engages the lock.*/
	hMplkRegs->MPLKCMD = 0x2;

	/*wait to make sure it is locked*/
	while(0==(hMplkRegs->MPLKSTAT&0x1));
}

/**
********************************************************************************
 *   @n@b void unlock_mem_prot_regs()
 *
 *   @b ܣ
 *   @n Դ洢Ĵ
 *
 *   @b βΣ
 *   @verbatim
 *      
 *   @endverbatim
 *
 *   <b> ֵ </b>
 *   @n 
 *
 *   <b> ע </b>
 *   @n 
 *******************************************************************************
 */
void unlock_mem_prot_regs()
{
	int i;

	/*1. Write a 1 to the KEYR field in the MPLKCMD register. This resets some internal
	status for the MPLK0 through the MPLK3 registers.*/
	hMplkRegs->MPLKCMD = 0x4;

	/*2. Write the unlock key to MPLK0 through the MPLK3 registers. The hardware
	compares the written value with the stored key value. Software must write to all
	four registers exactly once. The writes can arrive in any order.*/
	for(i=0; i<4; i++)
	{
		hMplkRegs->MPLK[i] = mem_prot_key[i];
	}

	/*3. Write a 1 to the UNLOCK field in the MPLKCMD register. If the key written in
	step 2 matches the stored key, the hardware disengages the lock. If the key written
	in step 2 does not match, the hardware signals an exception. The hardware
	reports the fault address as the address of the MPLKCMD register.*/
	hMplkRegs->MPLKCMD = 0x1;

	/*wait to make sure it is unlocked*/
	while(1==(hMplkRegs->MPLKSTAT&0x1));
}
